@@ -18,15 +18,12 @@ args = parser.parse_args()
 # Set active directory
 os.chdir(args.chdir)
 # Create PyQT5 application
-app = init_qt_app()
+app = init_qt_app(enable_hd_scaling=False)
 
 # Load configuration file
 with open(args.conf, "r") as configFile:
     config = yaml.load(configFile, Loader=yaml.Loader)
 
-# Start control loop for clients specified in configuration file
-kamzik3.session.start_client_poller_loop()
-
 signal.signal(signal.SIGINT, signal.SIG_DFL)
 if "session_window" in config:
     config["session_window"].show()
