@@ -1,6 +1,6 @@
 # Responses
-RESPONSE_OK = "1"
-RESPONSE_ERROR = "0"
+RESPONSE_OK = b"1"
+RESPONSE_ERROR = b"0"
 
 # Statuses
 STATUS_DISCONNECTED = "Disconnected"
@@ -44,6 +44,9 @@ OFFSET = "Offset"
 TOLERANCE = "Tolerance"
 SETPOINT = "Setpoint"
 ALL = "*"
+DISPLAY = "display"
+IMAGE = "Image"
+LOCAL_FILESYSTEM = "Local filesystem"
 
 # Basic attributes
 ATTR_ID = "Id"
@@ -51,16 +54,22 @@ ATTR_DESCRIPTION = "Description"
 ATTR_STATUS = "Status"
 ATTR_ENABLED = "Enabled"
 
-GET = "0"
-SET = "1"
-COMMAND = "2"
-INIT = "3"
-METHOD = "4"
-POLL = "5"
+INSTRUCTION_GET = b"0"
+INSTRUCTION_SET = b"1"
+INSTRUCTION_COMMAND = b"2"
+INSTRUCTION_INIT = b"3"
+INSTRUCTION_METHOD = b"4"
+INSTRUCTION_POLL = b"5"
+INSTRUCTION_PULL = b"6"
+INSTRUCTION_PUSH = b"7"
+INSTRUCTION_FORWARD = b"8"
+INSTRUCTION_SYNC = b"9"
 
 # Tokens
 TOKEN_ATTRIBUTE = "A!"
 TOKEN_ATTRIBUTE_REPLACE = "R!"
+TOKEN_ATTRIBUTE_DELETE = "D!"
+TOKEN_ATTRIBUTE_GROUP_DELETE = "GD!"
 TOKEN_ALL = "*"
 
 # Words
@@ -78,6 +87,20 @@ MACRO_EXECUTE_METHOD_STEP = u"Execute method"
 # Config enums
 CFG_TEMPLATE = 0
 
+# Zmq message tokens
+WORKER_READY = b"1"
+MSG_TERMINATE = b"0"
+WORKER_DONE = b"2"
+MSG_EMPTY = b""
+MSG_JSON = b"J"
+MSG_ARRAY = b"A"
+MSG_PICKLE = b"P"
+MSG_FILE = b"F"
+MSG_ACK = b"ACK"
+MSG_FORWARD = b"FRWD"
+MSG_ABORT = b"ABT"
+MSG_PROGRESS = b"PROG"
+
 # List of general Attributes
 ATTR_SERIAL_NUMBER = u"Serial number"
 ATTR_PRODUCT_TYPE = u"Product type"
@@ -242,6 +265,7 @@ ATTR_MAX_VOLTAGE = u"Max voltage"
 ATTR_OUTPUT_SWITCH = u"Output switch"
 ATTR_INTENSITY = u"Intensity"
 ATTR_RESOURCE_DIRECTORY = u"Resource directory"
+ATTR_CACHE_DIRECTORY = u"Cache directory"
 ATTR_SCAN_PREFIX = u"Scan prefix"
 ATTR_SCAN_COUNT = u"Scan count"
 ATTR_OUTPUT_FILENAME = u"Output filename"
@@ -365,3 +389,29 @@ ATTR_PROCESS_ID = u"Process ID"
 ATTR_MEMORY_USAGE = u"Memory usage"
 ATTR_CPU_USAGE = u"Cpu usage"
 ATTR_AVAILABLE_MEMORY = u"Available memory"
+ATTR_HOME_SWITCH_FLAG = u"Home switch flag"
+ATTR_HARDWARE_VERSION = u"Hardware version"
+ATTR_SOFTWARE_VERSION = u"Software version"
+ATTR_APPLICATION_NUMBER = u"Application number"
+ATTR_APPLICATION_VERSION = u"Application version"
+ATTR_OPERATION_MODE = u"Operation mode"
+ATTR_VELOCITY_UNITS = u"Velocity units"
+ATTR_WIDTH = u"Width"
+ATTR_HEIGHT = u"Height"
+ATTR_INPUT_DIR_LIST = u"Input dir list"
+ATTR_OUTPUT_DIR_LIST = u"Output dir list"
+ATTR_FILE_FILTER_PATTERN = u"File filter pattern"
+ATTR_ADDRESS = u"Address"
+ATTR_HOST = u"Host"
+ATTR_PORT = u"Port"
+ATTR_TASKS = u"Tasks"
+ATTR_WORKER = u"Worker"
+ATTR_CLIENT = u"Client"
+ATTR_TASK_INSTRUCTION = u"Task instruction"
+ATTR_SOURCE_FILE = u"Source file"
+ATTR_TARGET_FILE = u"Target file"
+ATTR_ABORTED = u"Aborted"
+ATTR_DRIVES = u"Drives"
+ATTR_FINISHED_AT = u"Finished at"
+ATTR_REQUESTED_FILES = u"Requested files"
+ATTR_SYNCED_FILES = u"Synced files"
