@@ -1,16 +1,15 @@
 import collections
 import math
-from threading import Lock
-from time import time
+from time import perf_counter_ns
 
 import pint
 from numpy import dtype, float16, float32, float64, uint16, uint32, uint64, int16, int32, int64, uint8, int8, ndarray
 
 from kamzik3 import units
 from kamzik3.constants import *
-
-
 # TODO: Deal with numpy arrays
+from kamzik3.snippets.snippetDataAccess import is_equal
+
 
 class SetFunction(object):
 
@@ -34,10 +33,12 @@ class Attribute(dict):
     float_types = (float16, float32, float64)
     unsigned_types = (uint8, uint16, uint32, uint64)
     with_set_function = False
-    last_update_time = time()
+    last_stream_time = last_update_time = perf_counter_ns()
+    last_update_time_diff = 0
     numerical = False
     set_token = 0
     set_callback = None
+    min_stream_timeout = 100 * 1e6
 
     def attribute_copy(self):
         return Attribute(**self)
@@ -46,7 +47,6 @@ class Attribute(dict):
                  set_function=None, default_type=u"str", decimals=0, readonly=False, description=None,
                  set_value_when_set_function=True, tolerance=(0, 0), factor=1, offset=0, **kwargs):
         dict.__init__(self)
-        self.update_lock = Lock()
         self._callbacks = {}
         self.attribute_id = attribute_id
         self.set_function = set_function
@@ -59,13 +59,14 @@ class Attribute(dict):
         if unit is None:
             unit = u""
         self[UNIT] = unit
+        self.min_stream_timeout = kwargs.get("min_stream_timeout", 100) * 1e6
         default_type = kwargs.get(TYPE, default_type)
         if default_type == TYPE_LIST:
             self[TYPE] = TYPE_LIST
         elif isinstance(default_type, (list, collections.abc.KeysView)):
             self[TYPE] = TYPE_LIST
             self[TYPE_LIST_VALUES] = list(default_type)
-        elif isinstance(default_type, ndarray):
+        elif default_type == TYPE_ARRAY or isinstance(default_type, ndarray):
             self[TYPE] = TYPE_ARRAY
         else:
             default_type = dtype(default_type)
@@ -92,9 +93,6 @@ class Attribute(dict):
         self.update(kwargs)
 
     def __setitem__(self, k, v):
-        if self.update_lock.locked():
-            return
-
         if k == VALUE:
             if isinstance(v, units.Quantity):
                 try:
@@ -111,16 +109,8 @@ class Attribute(dict):
                 if units.parse_unit_name(v) == ():
                     units.define(u"{}=1".format(v))
                     units.derived_units.append(v)
-                # try:
-                    # Try to get unit name from pint
-                    # print(v, units.parse_unit_name("degree"))
-                    # units.get_name(v)
-                # except pint.errors.UndefinedUnitError:
-                #     # Unit does not exists so it's added to the pint definitions
-                #     units.define(u"{}=1".format(v))
-                #     units.derived_units.append(v)
         try:
-            if self[k] == v:
+            if is_equal(self[k], v):
                 return
         except KeyError:
             pass
@@ -141,20 +131,24 @@ class Attribute(dict):
                 self.set_token = self.set_function(setpoint)
             if not self.set_value_when_set_function:
                 return
+            self._update_set_time()
         else:
             # Store actual attribute[k] value into dictionary
             super(Attribute, self).__setitem__(k, v)
+            self._update_set_time()
 
         self.handle_callbacks(k, v)
 
-    def handle_callbacks(self, k, v):
-        # Set time of last update to now
-        self.last_update_time = time()
+    def _update_set_time(self):
+        update_time = perf_counter_ns()
+        self.last_update_time_diff = update_time - self.last_update_time
+        self.last_update_time = update_time
 
+    def handle_callbacks(self, k, v):
         # Call all relevant callbacks
         for callback, (max_update_rate, updated_at, key_filter) in self._callbacks.copy().items():
             # Check update rate, continue if it's less than max_update_rate
-            if k == VALUE and max_update_rate is not None and (time() - updated_at) < max_update_rate:
+            if k == VALUE and max_update_rate is not None and (self.last_update_time - updated_at) <= max_update_rate:
                 continue
             # Check if key_filter is set
             if key_filter is None:
@@ -167,7 +161,7 @@ class Attribute(dict):
                 continue
             try:
                 # Set last callback time
-                self._callbacks[callback][1] = self.last_update_time
+                self._callbacks[callback][1] = perf_counter_ns()
             except KeyError:
                 # Callback no longer exists, skipping
                 pass
@@ -218,8 +212,8 @@ class Attribute(dict):
 
         if callback not in self._callbacks:
             if max_update_rate is not None:
-                max_update_rate *= 1e-3
-            self._callbacks[callback] = [max_update_rate, time(), key_filter]
+                max_update_rate *= 1e6
+            self._callbacks[callback] = [max_update_rate, perf_counter_ns(), key_filter]
 
     def detach_callback(self, callback):
         """
