@@ -4,12 +4,12 @@ from logging.handlers import TimedRotatingFileHandler
 
 import numpy as np
 
-from kamzik3 import DeviceError, DeviceUnknownError, units
+from kamzik3 import DeviceError, DeviceUnknownError
 from kamzik3.constants import *
 from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.device import Device
 from kamzik3.snippets.snippetLogging import set_rotating_file_handler, set_file_handler
-from kamzik3.snippets.snippetsTimer import CallbackTimer
+from kamzik3.snippets.snippetsTimer import PreciseCallbackTimer
 
 ATTR_LOGFILE = "Logfile"
 ATTR_HEADER = "Header"
@@ -32,6 +32,7 @@ class AttributeLogger(Device):
         if self.logged_attributes is None:
             self.logged_attributes = []
 
+        self.active_log_attributes = []
         self.log_file_name = log_file_name
         self.log_formatter = logging.Formatter(self.separator.join(['%(asctime)s', '%(created)s', '%(message)s']),
                                                datefmt='%Y-%m-%d %H:%M:%S')
@@ -58,29 +59,32 @@ class AttributeLogger(Device):
         self.timeout = timeout * 1e-3
 
     def _get_logged_attributes(self):
-        logged_values = []
-        value_unit = u""
-        for device_id, attribute in self.logged_attributes[:]:
+        logged_values = dict.fromkeys(self.active_log_attributes, str(np.NaN))
+        generate_header = False
+        for device_id, attribute in self.logged_attributes:
             try:
                 device = self.session.get_device(device_id)
-                logged_value, value_unit = None, u""
-                if device.in_statuses(READY_DEVICE_STATUSES):
-                    attribute = device.get_attribute(attribute)
-                    if attribute is not None and attribute.value() is not None:
-                        attribute_value = attribute.value()
-                        if isinstance(attribute_value, units.Quantity):
-                            logged_value = "{:~}".format(attribute.value())
-                        else:
-                            logged_value = attribute.value()
+                device_attribute = device.get_attribute(attribute)
+                logged_value = device_attribute[VALUE]
+                logged_type = device_attribute[TYPE]
+                logged_unit = device_attribute[UNIT]
+                if logged_type == "":
+                    logged_type = "str"
+                if logged_unit == "":
+                    logged_unit = "None"
+                header_lookup = "{} {} ({}, {})".format(device_id, attribute, logged_type, logged_unit)
+                if header_lookup not in self.active_log_attributes:
+                    self.active_log_attributes.append(header_lookup)
+                    generate_header = True
+                if logged_value is None:
+                    logged_value = np.NaN
+                logged_values[header_lookup] = str(logged_value)
             except DeviceUnknownError:
-                logged_value = None
+                continue
 
-            if logged_value is None:
-                logged_value, value_unit = u"None", u""
-
-            logged_values.append("{}{}".format(logged_value, value_unit))
-
-        return logged_values
+        if generate_header:
+            self.write_header()
+        return logged_values.values()
 
     def set_log_file_name(self, log_file_name):
         self.set_raw_value(ATTR_LOGFILE, log_file_name)
@@ -119,7 +123,8 @@ class AttributeLogger(Device):
         logged_values = self._get_logged_attributes()
         last_log_line = self.separator.join(logged_values)
         self.set_raw_value(ATTR_LAST_LOG_LINE, last_log_line)
-        self.attribute_logger.info(last_log_line)
+        if self.attribute_logger:
+            self.attribute_logger.info(last_log_line)
 
     def add_logged_attribute(self, device_id, attribute):
         attribute = Attribute.list_attribute(attribute)
@@ -137,9 +142,10 @@ class AttributeLogger(Device):
         self.logged_attributes.remove((device_id, attribute))
 
     def generate_header(self):
-        header_line = ["Datetime", "Timestamp"]
-        for device_id, attribute in self.logged_attributes[:]:
-            header_line.append(u"{} {}".format(device_id, attribute))
+        header_line = ["Datetime ({}, {})".format("str", "None"),
+                       "Timestamp ({}, {})".format(np.dtype("float").name, "None")]
+
+        header_line += self.active_log_attributes
         return "# " + self.separator.join(header_line).replace("'", "").replace(", ", ",")
 
     def write_header(self):
@@ -154,9 +160,7 @@ class AttributeLogger(Device):
             if self.logger_ticker is not None and self.is_status(STATUS_BUSY):
                 self.logger_ticker.stop()
                 raise DeviceError(u"Logging is already running")
-
-            self.logger_ticker = CallbackTimer(self.interval, self.handle_interval_timeout, with_correction=True)
-            self.write_header()
+            self.logger_ticker = PreciseCallbackTimer(self.interval, self.handle_interval_timeout, with_correction=True)
             self.set_status(STATUS_BUSY)
             self.logger_ticker.start()
         else:
@@ -197,7 +201,7 @@ class AttributeLoggerTriggered(AttributeLogger):
             if self.is_status(STATUS_BUSY):
                 raise DeviceError(u"Logging is already running")
 
-            self.write_header()
+            # self.write_header()
             self.set_status(STATUS_BUSY)
         else:
             self.stop()
