@@ -1,17 +1,19 @@
 import json
+import pickle
 import time
 from threading import Thread, Event, Lock
-from time import sleep
 
+import numpy as np
 import zmq
 from zmq.utils.monitor import recv_monitor_message
 
-from kamzik3 import SEP, DeviceClientError
+import kamzik3
+from kamzik3 import DeviceClientError
 from kamzik3.constants import *
 from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.device import Device
-from kamzik3.snippets.snippetDataAccess import get_from_dict, set_in_dict
-from kamzik3.snippets.snippetsJson import JsonKamzikEncoder
+from kamzik3.snippets.snippetDataAccess import get_from_dict, set_in_dict, is_equal
+from kamzik3.snippets.snippetsJson import JsonKamzikEncoder, JsonKamzikHook
 
 """ Example of yaml configuration
 DeviceClient0: &DeviceClient0 !Device:kamzik3.devices.deviceClient.DeviceClient
@@ -128,14 +130,14 @@ class DeviceClient(Device):
         :return: None
         """
         self.logger.info(u"Initializing {} client subscriber thread".format(self.device_id))
-        # Get SUB type of socker
+        # Get SUB type of socket
         self.subscriber_socket = zmq.Context.instance().socket(zmq.SUB)
         # Set receive timeout to 100 ms
         self.subscriber_socket.setsockopt(zmq.RCVTIMEO, 100)
         # Don't let socket linger after close
         # This is VERY IMPORTANT not to set to anything but 0
         self.subscriber_socket.setsockopt(zmq.LINGER, 0)
-        self.subscriber_socket.connect("tcp://{}:{}".format(self.host, self.subscriber_port))
+        self.subscriber_socket.connect("tcp://{}:{}".format(self.host, kamzik3.session.config.get("publisher_port")))
         # Start separate Thread to read data from publisher
         self.subscription_thread = Thread(target=self._subscriber_thread)
         self.subscription_thread.start()
@@ -150,9 +152,17 @@ class DeviceClient(Device):
         while not self.stopped.isSet():
             if self.subscriber_socket.poll(timeout=100):
                 # Try to read any message from publisher
-                token, data = self.subscriber_socket.recv().decode().split(SEP)
-                data = json.loads(data)
-                self.handle_readout(token, data)
+                reply = self.subscriber_socket.recv_multipart()
+                token, stype = reply[:2]
+                if stype == MSG_JSON:
+                    data = json.loads(reply[2].decode(), object_hook=JsonKamzikHook)
+                elif stype == MSG_ARRAY:
+                    dtype, shape = reply[2:4]
+                    reply[4] = np.frombuffer(reply[4], dtype=dtype.decode())
+                    data = np.reshape(reply[4], json.loads(shape.decode()))
+                elif stype == MSG_PICKLE:
+                    data = pickle.loads(reply[2])
+                self.handle_readout(token.decode(), data)
             elif self.monitor_socket.poll(timeout=0):
                 # Check if there is any message to read from monitor socket
                 message = recv_monitor_message(self.monitor_socket)
@@ -161,7 +171,7 @@ class DeviceClient(Device):
                     break
 
         # Client was disconnected for any reason, finish cleaning up and possibly reconnect
-        self.handle_disconnection()
+        self.handle_disconnect()
 
     def _subscribe(self, topic):
         """
@@ -229,7 +239,21 @@ class DeviceClient(Device):
                 if attribute_parts[-1] == TOKEN_ATTRIBUTE_REPLACE:
                     attribute_path = attribute_parts[2:-1]
                     attribute_path = self.attributes_sharing_map.get(attribute_id, attribute_path)
+
+                    sync_tree = self.attributes
+                    for part in attribute_path:
+                        if part not in sync_tree:
+                            sync_tree[part] = {}
+                        sync_tree = sync_tree[part]
                     self._set(attribute_path, Attribute.from_dict(data))
+                elif attribute_parts[-1] == TOKEN_ATTRIBUTE_DELETE:
+                    attribute_path = attribute_parts[2:-1]
+                    attribute_path = self.attributes_sharing_map.get(attribute_id, attribute_path)
+                    sync_tree = self.attributes[attribute_path[:-1]]
+                    del sync_tree[attribute_path[-1]]
+                elif attribute_parts[-1] == TOKEN_ATTRIBUTE_GROUP_DELETE:
+                    attribute_path = attribute_parts[2:-1]
+                    del self.attributes[attribute_path[0]]
                 else:
                     attribute_path = attribute_parts[2:]
 
@@ -249,12 +273,11 @@ class DeviceClient(Device):
         """
         try:
             with self.client_comm_lock:
-                self.socket.send(SEP.join((self.device_id, INIT)).encode())
+                self.socket.send_multipart([INSTRUCTION_INIT, self.device_id.encode()])
                 status, token, response = self.handle_server_response()
-
             if status == RESPONSE_OK:
-                self.subscriber_port, attributes, self.attributes_sharing_map, self.exposed_methods, self.qualified_name = json.loads(
-                    response)
+                attributes, self.attributes_sharing_map, self.exposed_methods, self.qualified_name = json.loads(
+                    response, object_hook=JsonKamzikHook)
                 self.initial_status = attributes[ATTR_STATUS][VALUE]
                 self._sync_attributes(attributes, self.attributes)
                 for method_name, attributes in self.exposed_methods[:]:
@@ -278,7 +301,7 @@ class DeviceClient(Device):
         :param synced_attributes: dict
         :return:
         """
-        # Walk thru all items of dictionary
+        # Walk through all items of dictionary
         for key, attribute in synced_attributes.items():
             self._sync_attribute(key, attribute, parent_attribute)
 
@@ -298,7 +321,7 @@ class DeviceClient(Device):
                 Prevent attribute changes when syncing with new values
                 """
                 for attr_key in attribute.keys():
-                    if parent_attribute[key][attr_key] != attribute[attr_key]:
+                    if not is_equal(parent_attribute[key][attr_key], attribute[attr_key]):
                         parent_attribute[key][attr_key] = attribute[attr_key]
             else:
                 # Attribute is only sub group, continue syncing deeper
@@ -329,7 +352,7 @@ class DeviceClient(Device):
         try:
             attribute_value = json.dumps((attribute, value), ensure_ascii=True)
             with self.client_comm_lock:
-                self.socket.send(SEP.join((self.device_id, SET, attribute_value)).encode())
+                self.socket.send_multipart([INSTRUCTION_SET, self.device_id.encode(), attribute_value.encode()])
                 status, token, response = self.handle_server_response()
             if status == RESPONSE_OK:
                 if callback is not None:
@@ -380,8 +403,14 @@ class DeviceClient(Device):
         :return: None
         """
         current_value = get_from_dict(self.attributes, attribute)
-        if value != current_value:
+        if not is_equal(value, current_value):
             set_in_dict(self.attributes, attribute, value)
+        # try:
+        #     if value != current_value:
+        #         set_in_dict(self.attributes, attribute, value)
+        # except ValueError:
+        #     if not np.array_equal(value, current_value):
+        #         set_in_dict(self.attributes, attribute, value)
 
     def poll(self):
         """
@@ -393,7 +422,7 @@ class DeviceClient(Device):
                 u"Remote server poll failed. Device client is not connected")
         try:
             with self.client_comm_lock:
-                self.socket.send(SEP.join((self.device_id, POLL)).encode())
+                self.socket.send_multipart([INSTRUCTION_POLL, self.device_id.encode()])
                 status, token, response = self.handle_server_response()
             if status == RESPONSE_OK:
                 return response
@@ -419,15 +448,14 @@ class DeviceClient(Device):
 
         try:
             with self.client_comm_lock:
-                self.socket.send(SEP.join((self.device_id, COMMAND, command, "1" if callback else "0")).encode())
+                self.socket.send_multipart(
+                    [INSTRUCTION_COMMAND, self.device_id.encode(), command.encode(), b"1" if callback else b"0"])
                 status, token, response = self.handle_server_response()
-
             if status == RESPONSE_OK:
                 if token and callback is not None:
                     self.set_callback(self.get_token(token), callback)
             else:
                 raise DeviceClientError(u"Failed to execute remote command '{}'\n{}".format(command, response))
-
             return response
         except (zmq.Again, zmq.ZMQError) as e:
             self.handle_response_error(u"Command error: {}".format(e))
@@ -447,10 +475,11 @@ class DeviceClient(Device):
         try:
             dumped_attributes = json.dumps(attributes, ensure_ascii=True, cls=JsonKamzikEncoder)
             with self.client_comm_lock:
-                self.socket.send(SEP.join((self.device_id, METHOD, str(method), dumped_attributes)).encode())
+                self.socket.send_multipart(
+                    [INSTRUCTION_METHOD, self.device_id.encode(), str(method).encode(), dumped_attributes.encode()])
                 status, token, response = self.handle_server_response()
             if status == RESPONSE_OK:
-                return json.loads(response)
+                return json.loads(response, object_hook=JsonKamzikHook)
             else:
                 raise DeviceClientError(u"Failed to execute remote method '{}'\n{}".format(method, response))
 
@@ -464,9 +493,12 @@ class DeviceClient(Device):
         If server takes more then RCVTIMEO time, then we get zmq error.
         In that case modify server response time or increase RCVTIMEO time.
         '''
-        response = self.socket.recv().decode()
-        status, token, response = response.split(SEP)
-        return status, token, response
+        message = self.socket.recv_multipart()
+        status, token, msg_type = message[:3]
+        if msg_type == MSG_JSON:
+            return status, token.decode(), message[3].decode()
+        elif msg_type == MSG_FILE:
+            return status, token.decode(), message[3].decode()
 
     def handle_configuration(self):
         start_at = time.time()
@@ -501,7 +533,7 @@ class DeviceClient(Device):
         self.set_status(STATUS_DISCONNECTING)
         self.stopped.set()
 
-    def handle_disconnection(self):
+    def handle_disconnect(self):
         if self.logger is not None:
             self.logger.info(u"Client closed")
         if not self.removing:
