@@ -30,16 +30,24 @@ class DeviceSession(Device):
     def __init__(self, device_id=None, config=None):
         kamzik3.session = self
         self.devices = OrderedDict()
+        self.server = config.get("server")
+        self.publisher = config.get("publisher")
         self.measurement_groups = {}
         self.polling_timer = CallbackTimer(1000, self.measure_cpu_ram)
         self.os_process = psutil.Process(os.getpid())
         super(DeviceSession, self).__init__(device_id, config)
         self.connect()
 
+        if self.server is not None:
+            self.server.start()
+        if self.publisher is not None:
+            self.publisher.start()
+
     def _init_attributes(self):
         super(DeviceSession, self)._init_attributes()
         self.create_attribute(ATTR_LOG_DIRECTORY, set_function=self.set_log_directory, readonly=True)
         self.create_attribute(ATTR_RESOURCE_DIRECTORY, set_function=self.set_resource_directory, readonly=True)
+        self.create_attribute(ATTR_CACHE_DIRECTORY, set_function=self.set_cache_directory, readonly=True)
         self.create_attribute(ATTR_DEVICES_COUNT, default_value=0, default_type=np.uint64, readonly=True)
         self.create_attribute(ATTR_ALLOW_CONSOLE_LOG, default_value=True, default_type=np.bool)
         self.create_attribute(ATTR_ALLOW_GUI_LOG, default_value=True, default_type=np.bool)
@@ -48,8 +56,10 @@ class DeviceSession(Device):
         self.create_attribute(ATTR_ALLOW_MACRO_LOG, default_value=True, default_type=np.bool)
         self.create_attribute(ATTR_LOOPS_RUNNING, default_value=False, default_type=np.bool, readonly=True)
         self.create_attribute(ATTR_PROCESS_ID, default_value=os.getpid(), readonly=True)
-        self.create_attribute(ATTR_CPU_USAGE, default_value=0, default_type=np.float, readonly=True, unit="%", decimals=4)
-        self.create_attribute(ATTR_MEMORY_USAGE, default_value=0, default_type=np.float, readonly=True, decimals=3, unit="MB", factor=10e-7)
+        self.create_attribute(ATTR_CPU_USAGE, default_value=0, default_type=np.float, readonly=True, unit="%",
+                              decimals=4)
+        self.create_attribute(ATTR_MEMORY_USAGE, default_value=0, default_type=np.float, readonly=True, decimals=3,
+                              unit="MB", factor=10e-7)
         self.create_attribute(ATTR_AVAILABLE_MEMORY, default_value=0, default_type=np.float,
                               readonly=True, unit="MB", factor=10e-7, decimals=3)
         self.set_raw_value(ATTR_AVAILABLE_MEMORY, psutil.virtual_memory().total)
@@ -85,6 +95,8 @@ class DeviceSession(Device):
                 self._config_attributes_log()
             if self.get_value(ATTR_ALLOW_MACRO_LOG):
                 self.set_macro_log_directory(u"Macro")
+            if self.get_value(ATTR_CACHE_DIRECTORY) is None:
+                self.set_value(ATTR_CACHE_DIRECTORY, ".cache")
 
             self.start_polling()
             self.set_status(STATUS_CONFIGURED)
@@ -133,10 +145,7 @@ class DeviceSession(Device):
             self.devices[device.device_id] = device
             device.session = self
             self.set_value(ATTR_DEVICES_COUNT, len(self.devices))
-            if self.device_server is not None:
-                self.device_server.add_device(device)
         else:
-
             raise DeviceError("Device {} is already registered within session.".format(device.device_id))
 
     def remove_device(self, device):
@@ -188,6 +197,12 @@ class DeviceSession(Device):
             self.logger.info(u"Directory {} does not exists, trying to create it.".format(value))
             os.makedirs(value)
 
+    def set_cache_directory(self, value):
+        self.logger.info(u"Setting cache directory to: {}".format(value))
+        if not os.path.exists(value):
+            self.logger.info(u"Directory {} does not exists, trying to create it.".format(value))
+            os.makedirs(value)
+
     def set_console_log_directory(self, value):
         log_directory = self.get_value(ATTR_LOG_DIRECTORY)
         if log_directory is None:
@@ -300,9 +315,12 @@ class DeviceSession(Device):
 
     def stop(self):
         self.stop_polling()
-        
-        if self.device_server is not None:
-            self.device_server.close()
+
+        if self.server is not None:
+            self.server.stop()
+
+        if self.publisher is not None:
+            self.publisher.stop()
 
         self.session = None
         del self.devices[self.device_id]
