@@ -6,6 +6,7 @@ import serial
 from bidict import frozenbidict
 from pint.errors import UndefinedUnitError
 
+import kamzik3
 from kamzik3 import DeviceUnitError
 from kamzik3.constants import *
 from kamzik3.devices.deviceChannel import DeviceChannel
@@ -152,19 +153,19 @@ class DeviceSmaractMcs1(DeviceSocket):
         if callback is not None:
             self.handle_readout_callback(callback, command, readout_buffer)
         if token:
-            self.device_server.push_message(self.device_id, [command, readout_buffer], token)
+            kamzik3.session.publisher.push_message(self.device_id, [command, readout_buffer], token)
             for observer in self._observers[:]:
                 if isinstance(observer, DeviceSmaractMcs1Channel):
-                    self.device_server.push_message(observer.device_id, [command, readout_buffer], token)
+                    kamzik3.session.publisher.push_message(observer.device_id, [command, readout_buffer], token)
 
         # if callback is not None:
         #     self.handle_readout_callback(callback, command, readout_buffer)
         # if token:
-        #     self.device_server.push_message(self.device_id, readout_buffer, token)
+        #     kamzik3.session.publisher.push_message(self.device_id, readout_buffer, token)
         #
         #     for observer in self._observers[:]:
         #         if isinstance(observer, DeviceSmaractMcs1Channel):
-        #             self.device_server.push_message(observer.device_id, readout_buffer, token)
+        #             kamzik3.session.publisher.push_message(observer.device_id, readout_buffer, token)
 
     def set_hand_tool_mode(self, value):
         self.logger.info(u"Set hand tool mode to {}".format(value))
@@ -497,7 +498,7 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                                                int(max_angular_position), int(max_revolutions)), callback,
                 with_token=True)
 
-        self.set_attribute([ATTR_POSITION, MIN], self[ATTR_POSITION].apply_offset_factor(value))
+        self.set_raw_value([ATTR_POSITION, MIN], value)
         self.get_position_limit_range()
         return out
 
@@ -522,7 +523,7 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                                                int(max_angular_position), int(max_revolutions)), callback,
                 with_token=True)
 
-        self.set_attribute([ATTR_POSITION, MAX], self[ATTR_POSITION].apply_offset_factor(value))
+        self.set_raw_value([ATTR_POSITION, MAX], value)
         self.get_position_limit_range()
         return out
 
