@@ -130,7 +130,8 @@ class DeviceTango(Device, DeviceProxy):
             else:
                 value = str(value)
 
-            self.create_attribute(attribute, default_value=value, readonly=not ta.nb_written, description=tc.description,
+            self.create_attribute(attribute, default_value=value, readonly=not ta.nb_written,
+                                  description=tc.description,
                                   default_type=default_type, set_function=set_function, unit=tc.unit,
                                   min_value=min_value, max_value=max_value, decimals=decimals)
 
