@@ -196,8 +196,8 @@ class ScanAttributeWidget(Ui_Form, MacroWidget):
             row += 1
             self.macro_widgets.append(self._show_positive_tolerance_widget(row, user_unit))
             row += 1
-            # self.macro_widgets.append(self._show_random_deviation_widget(row))
-            # row += 1
+            self.macro_widgets.append(self._show_random_deviation_widget(row))
+            row += 1
             self.macro_widgets.append(self._show_wait_after_widget(row))
             row += 1
             self.macro_widgets.append(self._show_repeat_widget(row))
@@ -450,7 +450,7 @@ class ScanAttributeWidget(Ui_Form, MacroWidget):
                             scanner_attributes=scanner_attributes, return_back=self.config.get("return_back", True))
             else:
                 scan = Scan("Scan", values["start_point"], values["end_point"], int(values["total_steps"].m),
-                            repeat_count=values["repeat"], step_class=StepDeviceAttributeNumerical,
+                            repeat_count=values["repeat"], random_deviation=values["random_deviation"], step_class=StepDeviceAttributeNumerical,
                             step_attributes=step_attributes, as_step=False,
                             bidirectional=self.config.get("bidirectional", False), scanner=scanner,
                             scanner_attributes=scanner_attributes, return_back=self.config.get("return_back", True))
