@@ -1,6 +1,7 @@
 import argparse
 import os
 import signal
+
 import oyaml as yaml
 
 import kamzik3
@@ -17,7 +18,7 @@ args = parser.parse_args()
 # Set active directory
 os.chdir(args.chdir)
 # Create PyQT5 application
-app = init_qt_app()
+app = init_qt_app(enable_hd_scaling=False)
 
 # Load configuration file
 with open(args.conf, "r") as configFile:
@@ -30,6 +31,9 @@ signal.signal(signal.SIGINT, signal.SIG_DFL)
 if "session_window" in config:
     config["session_window"].show()
 
+
+# config["logView"].show()
+# config["logView"].open("/home/domarm/PycharmProjects/mag2/mag2/server_log/Attributes/attributes.log.2020-05-08")
 # Start PyQT5 app loop
 app.exec_()
 
